{

    Common.pp                          last modified: 5 July 2014

    Copyright (C) Jaroslaw Baran, furious programming 2011 - 2014.
    All rights reserved.
   __________________________________________________________________________

    This unit is a part of the SnakeASCII console game.
   __________________________________________________________________________

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
   __________________________________________________________________________

}

unit Common;

{$MODE OBJFPC}{$H+}{$PACKENUM 1}

interface

uses
  Crt;


{ ----- game -------------------------------------------------------------------------------------------------- }

{ ----- types ------------------------- }

type
  { single module coordinates record }
  TModuleCoords = record
    X, Y: UInt8;
  end;

type
  { move direction enumeration }
  TMoveDirection = (mdUp, mdDown, mdLeft, mdRight, mdNone);

type
  { lenght of snake type }
  TSnakeLength = UInt16;

{ ----- constants ------------------------- }

const
  { level bounds values }
  LBV_LEFT   = UInt8(1);
  LBV_RIGHT  = UInt8(65);
  LBV_TOP    = UInt8(1);
  LBV_BOTTOM = UInt8(15);

const
  { map coordinates offsets }
  MCO_HORZ = UInt8(7);
  MCO_VERT = UInt8(6);

const
  { snake default length }
  SNAKE_INITIAL_LENGTH = TSnakeLength(3);

const
  { delay time for single game frame }
  SINGLE_FRAME_DELAY_TIME = UInt8(40);
  { delay time for drawing level bounds }
  LEVEL_DRAWING_DELAY_TIME = UInt8(10);
  { delay time for start the game }
  START_GAME_DELAY_TIME = UInt16(500);

const
  { constant for main game loop condition }
  SNAKE_IS_ALIVE = True;


{ ----- snake ------------------------------------------------------------------------------------------------- }

{ ----- types ------------------------- }

type
  { snake single module basic type enumeration }
  TSnakeModuleBasicType = (
    smbtNone            = 0,   { none module                 }
    smbtVertical        = 186, { vertical body               }
    smbtCornerLeftDown  = 187, { corner from bottom to left  }
    smbtCornerLeftUp    = 188, { corner from top to left     }
    smbtCornerRightUp   = 200, { corner from top to right    }
    smbtCornerRightDown = 201, { corner from bottom to right }
    smbtHorzizontal     = 205, { horizontal body             }
    smbtHead            = 219  { head module                 }
  );

type
  { snake single module additional type enumeration }
  TSnakeModuleAdditionalType = (
    smatLeftVert   = 185, { gate from vertical to left     }
    smatTopHorz    = 202, { gate from horizontal to top    }
    smatBottomHorz = 203, { gate from horizontal to bottom }
    smatRightVert  = 204, { gate from vertical to right    }
    smatCrossGate  = 206  { cross gate                     }
  );

type
  { snake single module record }
  TSnakeModuleRec = record
    ModuleType: TSnakeModuleBasicType;
    Coords: TModuleCoords;
    GoldModule: Boolean;
  end;

{ ----- constants ------------------------- }

const
  { snake high module index }
  SNAKE_HIGH_MODULE_INDEX = (LBV_RIGHT - LBV_LEFT - 1) * (LBV_BOTTOM - LBV_TOP - 1) - 1;

{ ----- types ------------------------- }

type
  { array of snake modules }
  TSnakeModulesArr = array [0 .. SNAKE_HIGH_MODULE_INDEX] of TSnakeModuleRec;

{ ----- constants ------------------------- }

const
  { live snake modules colors }
  SNAKE_COLORATION_LIVE_HEAD_TAIL_SHADE = UInt8(Yellow);
  { dead snake modules colors }
  SNAKE_COLORATION_DEAD_HEAD_TAIL_SHADE = UInt8(White);

const
  { snake directions characters bounds }
  SNAKE_MIN_MOVE_DIR_CHAR = AnsiChar('A');
  SNAKE_MAX_MOVE_DIR_CHAR = AnsiChar('D');

const
  { snake directions enumerations array }
  SNAKE_MOVE_DIRECTIONS_ARR: array [SNAKE_MIN_MOVE_DIR_CHAR .. SNAKE_MAX_MOVE_DIR_CHAR] of TMoveDirection = (
    mdRight, { 'A' -> move right }
    mdDown,  { 'B' -> move down  }
    mdLeft,  { 'C' -> move left  }
    mdUp     { 'D' -> move up    }
  );


{ ----- level ------------------------------------------------------------------------------------------------- }

{ ----- types ------------------------- }

type
  { level of difficulty enumeration }
  TLevelOfDifficulty = (lofEasy, lofMedium, lofHard, lofExtreme);

type
  { snake coloration enumeration }
  TSnakeColoration = (scGreen, scCyan, scBlue, scMagenta, scRed, scGold);

type
  { snake coloration array }
  TSnakeColorationArr = array [Boolean] of UInt8;

type
  { level of snake speed enumeration }
  TLevelOfSnakeSpeed = (lossImpossible = 2, lossExtreme = 3, lossHigh = 4, lossMedium = 5, lossLow = 6);

type
  { level map array }
  TLevelMapArr = array [LBV_LEFT .. LBV_RIGHT, LBV_TOP .. LBV_BOTTOM] of AnsiChar;

type
  { level hole record }
  TLevelHoleRec = record
    Left, Right, Top, Bottom: UInt8;
  end;

type
  { level holes array }
  TLevelHolesArr = array of TLevelHoleRec;

type
  { level grass clump record }
  TLevelGrassClumpRec = record
    Color: UInt8;
    Left, Right, Top, Bottom: UInt8;
  end;

type
  { level grass clumps array }
  TLevelGrassClumpsArr = array of TLevelGrassClumpRec;

{ ----- constants ------------------------- }

const
  { bounds color character difference }
  LEVEL_BOUNDS_COLOR_CHAR_DIFF = UInt8(64);
  { grass color character diference }
  LEVEL_GRASS_COLOR_CHAR_DIFF = UInt8(64);


const
  { level of difficulty characters bounds }
  LEVEL_MIN_DIF_CHAR = AnsiChar('A');
  LEVEL_MAX_DIF_CHAR = AnsiChar('D');

const
  { level of difficulty enumerations array }
  LEVEL_DIF_ENUMS_ARR: array [LEVEL_MIN_DIF_CHAR .. LEVEL_MAX_DIF_CHAR] of TLevelOfDifficulty = (
    lofEasy,   { 'A' -> easy    }
    lofMedium, { 'B' -> medium  }
    lofHard,   { 'C' -> hard    }
    lofExtreme { 'D' -> extreme }
  );

const
  { snake coloration characters bounds }
  SNAKE_COLORATION_MIN_CHAR = AnsiChar('A');
  SNAKE_COLORATION_MAX_CHAR = AnsiChar('F');

const
  { snake coloration enumerations array }
  SNAKE_COLORATION_ENUMS_ARR: array [SNAKE_COLORATION_MIN_CHAR .. SNAKE_COLORATION_MAX_CHAR] of TSnakeColoration = (
    scGreen,    { 'A' -> green snake   }
    scCyan,     { 'C' -> cyan snake    }
    scBlue,     { 'B' -> blue snake    }
    scMagenta,  { 'D' -> magenta snake }
    scRed,      { 'E' -> red snake     }
    scGold      { 'F' -> gold snake    }
  );

const
  { snake coloration colors shades for live snake }
  SNAKE_COLORATION_LIVE_SHADES_ARR: array [TSnakeColoration] of TSnakeColorationArr = (
    (Green,   LightGreen),    { 'A' -> green shades   }
    (Cyan,    LightCyan),     { 'C' -> cyan shades    }
    (Blue,    LightBlue),     { 'B' -> blue shades    }
    (Magenta, LightMagenta),  { 'D' -> magenta shades }
    (Red,     LightRed),      { 'E' -> red shades     }
    (Brown,   Yellow)         { 'F' -> gold shades    }
  );

const
  { snake coloration color shades for dead snake }
  SNAKE_COLORATION_DEAD_SHADES_ARR: TSnakeColorationArr = (DarkGray, LightGray);

const
  { snake speed characters bounds }
  SNAKE_SPEED_MIN_CHAR = AnsiChar('A');
  SNAKE_SPEED_MAX_CHAR = AnsiChar('E');

const
  { snake speed cycles array }
  SNAKE_SPEED_ENUMS_ARR: array [SNAKE_SPEED_MIN_CHAR .. SNAKE_SPEED_MAX_CHAR] of TLevelOfSnakeSpeed = (
    lossLow,        { 'A' -> low speed     }
    lossMedium,     { 'B' -> medium speed  }
    lossHigh,       { 'C' -> high speed    }
    lossExtreme,    { 'D' -> extreme speed }
    lossImpossible  { 'E' -> impossible    }
  );

const
  { level map empty element character }
  LEVEL_MAP_EMPTY = AnsiChar(' ');

  { level map line characters }
  LEVEL_MAP_HORZ_LINE = AnsiChar(#196);
  LEVEL_MAP_VERT_LINE = AnsiChar(#179);

  { level map corner characters }
  LEVEL_MAP_CORNER_UP_LEFT    = AnsiChar(#217);
  LEVEL_MAP_CORNER_UP_RIGHT   = AnsiChar(#192);
  LEVEL_MAP_CORNER_DOWN_LEFT  = AnsiChar(#191);
  LEVEL_MAP_CORNER_DOWN_RIGHT = AnsiChar(#218);

  { level map T-section characters }
  LEVEL_MAP_TSECTION_HORZ_UP    = AnsiChar(#193);
  LEVEL_MAP_TSECTION_HORZ_DOWN  = AnsiChar(#194);
  LEVEL_MAP_TSECTION_VERT_LEFT  = AnsiChar(#180);
  LEVEL_MAP_TSECTION_VERT_RIGHT = AnsiChar(#195);

  { level map cross character }
  LEVEL_MAP_CROSS = AnsiChar(#197);

const
  { level map characters bounds }
  LEVEL_MIN_MAP_CHAR = AnsiChar('`');
  LEVEL_MAX_MAP_CHAR = AnsiChar('k');

const
  { level map characters array }
  LEVEL_MAP_CHARS_ARR: array [LEVEL_MIN_MAP_CHAR .. LEVEL_MAX_MAP_CHAR] of AnsiChar = (
    LEVEL_MAP_EMPTY,               { '`' -> space                               }
    LEVEL_MAP_HORZ_LINE,           { 'a' -> horizontal line                     }
    LEVEL_MAP_VERT_LINE,           { 'b' -> vertical line                       }
    LEVEL_MAP_CORNER_DOWN_RIGHT,   { 'c' -> corner from bottom to right         }
    LEVEL_MAP_CORNER_DOWN_LEFT,    { 'd' -> corner from bottom to left          }
    LEVEL_MAP_CORNER_UP_LEFT,      { 'e' -> corner from top to left             }
    LEVEL_MAP_CORNER_UP_RIGHT,     { 'f' -> corner from top to right            }
    LEVEL_MAP_TSECTION_VERT_RIGHT, { 'g' -> T-section from vertical to right    }
    LEVEL_MAP_TSECTION_HORZ_DOWN,  { 'h' -> T-section from horizontal to bottom }
    LEVEL_MAP_TSECTION_VERT_LEFT,  { 'i' -> T-section from vertical to left     }
    LEVEL_MAP_TSECTION_HORZ_UP,    { 'j' -> T-section from horizontal to top    }
    LEVEL_MAP_CROSS                { 'k' -> cross                               }
  );

const
  { grass clump characters count }
  GRASS_CLUMP_CHARS_COUNT = UInt8(4);

const
  { grass clump available characters }
  GRASS_CLUMP_CHARS_ARR: array [0 .. GRASS_CLUMP_CHARS_COUNT - 1] of AnsiChar = ('v', 'V', 'w', 'W');

const
  { level file section name }
  LFSN_INFO         = AnsiString('INFO');
  LFSN_SNAKE        = AnsiString('SNAKE');
  LFSN_MAP          = AnsiString('MAP');
  LFSN_HOLES        = AnsiString('HOLES');
  LFSN_HOLE         = AnsiString('HOLE');
  LFSN_GRASS_CLUMPS = AnsiString('GRASS CLUMPS');
  LFSN_GRASS_CLUMP  = AnsiString('CLUMP');

const
  { level file key name - info }
  LFKN_INFO_NAME       = AnsiString('Name');
  LFKN_INFO_DIFFICULTY = AnsiString('Difficulty');

  { level file key name - snake }
  LFKN_SNAKE_COLORATION     = AnsiString('Coloration');
  LFKN_SNAKE_SPEED          = AnsiString('Speed');
  LFKN_SNAKE_DEF_HEAD_POS_X = AnsiString('HeadPosX');
  LFKN_SNAKE_DEF_HEAD_POS_Y = AnsiString('HeadPosY');
  LFKN_SNAKE_DEF_DIRECTION  = AnsiString('Direction');

  { level file key name - map }
  LFKN_MAP_BOUNDS_COLOR = AnsiString('BoundsColor');

  { level file key name - holes }
  LFKN_HOLES_COUNT = AnsiString('Count');

  { level file key name - hole }
  LFKN_HOLE_LEFT   = AnsiString('Left');
  LFKN_HOLE_RIGHT  = AnsiString('Right');
  LFKN_HOLE_TOP    = AnsiString('Top');
  LFKN_HOLE_BOTTOM = AnsiString('Bottom');

  { level file key name - grass clumps }
  LFKN_CLUMPS_COUNT = AnsiString('Count');

  { level file key name - grass clump }
  LFKN_CLUMP_COLOR  = AnsiString('Color');
  LFKN_CLUMP_LEFT   = AnsiString('Left');
  LFKN_CLUMP_RIGHT  = AnsiString('Right');
  LFKN_CLUMP_TOP    = AnsiString('Top');
  LFKN_CLUMP_BOTTOM = AnsiString('Bottom');


{ ----- gold -------------------------------------------------------------------------------------------------- }

{ ----- constants ------------------------- }

const
  { gold symbol and color }
  GOLD_SYMBOL = Char(#4);
  GOLD_COLOR  = UInt8(Yellow);

const
  { minimum difference between previous and new gold coordinates }
  RANDOM_GOLD_MIN_DIFF = UInt8(2);

{ ----- types ------------------------- }

type
  { gold coordinates validation functions }
  TIsInLevelProc    = function(ACoords: TModuleCoords): Boolean of object;
  TIsNotInSnakeProc = function(ACoords: TModuleCoords): Boolean of object;


{ ----- timer ------------------------------------------------------------------------------------------------- }


const
  { timer zero step value }
  TIMER_ZERO_STEP = UInt8(0);


{ ----- controller -------------------------------------------------------------------------------------------- }

{ ----- types ------------------------- }

type
  { double-byte type for input key codes }
  TInputKey = UInt16;

{ ----- constants ------------------------- }

const
  { input standard keys count }
  INPUT_STD_KEYS_COUNT = UInt8(5);
  { input extended keys count }
  INPUT_EXT_KEYS_COUNT = UInt8(3);

const
  { input key code prefix for standard key }
  INPUT_STD_KEY_CODE_PREFIX = TInputKey($0000);
  { input key code prefix for special key }
  INPUT_EXT_KEY_CODE_PREFIX = TInputKey($FF00);

const
  { no key pressed or not supported key value }
  INPUT_KEY_NONE = TInputKey($0000);

const
  { input keys for snake moves }
  INPUT_DEFAULT_KEY_UP    = TInputKey(INPUT_EXT_KEY_CODE_PREFIX or $0048);  { arrow up    - move up    }
  INPUT_DEFAULT_KEY_DOWN  = TInputKey(INPUT_EXT_KEY_CODE_PREFIX or $0050);  { arrow down  - move down  }
  INPUT_DEFAULT_KEY_LEFT  = TInputKey(INPUT_EXT_KEY_CODE_PREFIX or $004B);  { arrow left  - move left  }
  INPUT_DEFAULT_KEY_RIGHT = TInputKey(INPUT_EXT_KEY_CODE_PREFIX or $004D);  { arrow right - move right }
  INPUT_DEFAULT_KEY_TURBO = TInputKey(INPUT_STD_KEY_CODE_PREFIX or $0020);  { space       - turbo mode }

const
  { input special keys }
  INPUT_DEFAULT_KEY_PAUSE = TInputKey(INPUT_STD_KEY_CODE_PREFIX or $0050);  { P   - pause the game }
  INPUT_DEFAULT_KEY_RESET = TInputKey(INPUT_STD_KEY_CODE_PREFIX or $0052);  { R   - reset the game }
  INPUT_DEFAULT_KEY_QUIT  = TInputKey(INPUT_STD_KEY_CODE_PREFIX or $001B);  { Esc - quit the game  }

const
  { input keys set for snake move }
  INPUT_SNAKE_MOVE_KEYS_ARR: array [0 .. INPUT_STD_KEYS_COUNT - 1] of TInputKey = (
    INPUT_DEFAULT_KEY_UP,     { move up    }
    INPUT_DEFAULT_KEY_DOWN,   { move down  }
    INPUT_DEFAULT_KEY_LEFT,   { move left  }
    INPUT_DEFAULT_KEY_RIGHT,  { move right }
    INPUT_DEFAULT_KEY_TURBO   { turbo mode }
  );

const
  { input special keys set }
  INPUT_SPECIAL_KEYS_ARR: array [0 .. INPUT_EXT_KEYS_COUNT - 1] of TInputKey = (
    INPUT_DEFAULT_KEY_PAUSE,  { pause the game }
    INPUT_DEFAULT_KEY_RESET,  { reset the game }
    INPUT_DEFAULT_KEY_QUIT    { quit the game  }
  );


{ ----- end interface ----------------------------------------------------------------------------------------- }


implementation


{ ----- end implementation ------------------------------------------------------------------------------------ }


end.

