{

    Controller.pp                      last modified: 2 July 2014

    Copyright (C) Jaroslaw Baran, furious programming 2011 - 2014.
    All rights reserved.
   __________________________________________________________________________

    This unit is a part of the SnakeASCII console game.
   __________________________________________________________________________

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
   __________________________________________________________________________

}

unit Controller;

{$MODE OBJFPC}{$H+}

interface

uses
  Crt, Common;


type
  { main game controller class }
  TController = class(TObject)
  private
    FInputKey: TInputKey;
  private
    function GetInputKeyPressedState(): Boolean;
    function ValidateInputKey(AKeysArray: array of TInputKey): Boolean;
  public
    constructor Create();
    destructor Destroy(); override;
  public
    procedure ProcessInput();
    procedure ResetInput();
  public
    property InputKey: TInputKey read FInputKey;
    property InputKeyPressed: Boolean read GetInputKeyPressedState;
  end;


{ ----- end interface ----------------------------------------------------------------------------------------- }


implementation


constructor TController.Create();
begin
  inherited Create();

  { initialize input key }
  FInputKey := INPUT_KEY_NONE;
end;


destructor TController.Destroy();
begin
  inherited Destroy();
end;


function TController.GetInputKeyPressedState(): Boolean;
begin
  { check the input key value }
  Result := FInputKey <> INPUT_KEY_NONE;
end;


function TController.ValidateInputKey(AKeysArray: array of TInputKey): Boolean;
var
  intToken: UInt8;
begin
  { find the same key in valid keys array }
  for intToken := Low(AKeysArray) to High(AKeysArray) do
    { compare keys }
    if FInputKey = AKeysArray[intToken] then
      Exit(True);

  { given key not found }
  Result := False;
end;


procedure TController.ProcessInput();
var
  chrInput: AnsiChar;
begin
  { check if the key was pressed }
  if KeyPressed() then
  begin
    { get the key from input }
    chrInput := UpCase(ReadKey());

    { detect extended key }
    if (Ord(FInputKey) = INPUT_KEY_NONE) and KeyPressed() then
    begin
      { get the extended key character }
      chrInput := UpCase(ReadKey());
      { build the extended key code }
      FInputKey := INPUT_EXT_KEY_CODE_PREFIX or Ord(chrInput);
    end
    else
      { build the standard key code }
      FInputKey := INPUT_STD_KEY_CODE_PREFIX or Ord(chrInput);

    { clear input buffer }
    while KeyPressed() do
      ReadKey();

    { validate input key }
    if ValidateInputKey(INPUT_SNAKE_MOVE_KEYS_ARR) or ValidateInputKey(INPUT_SPECIAL_KEYS_ARR) then
      Exit;
  end;

  { if key is not supported or key is not pressed - set the input null key }
  FInputKey := INPUT_KEY_NONE;
end;


procedure TController.ResetInput();
begin
  { set the input null key }
  FInputKey := INPUT_KEY_NONE;
end;


{ ----- end implementation ------------------------------------------------------------------------------------ }


end.

