{

    Converter.pp                       last modified: 2 July 2014

    Copyright (C) Jaroslaw Baran, furious programming 2011 - 2014.
    All rights reserved.
   __________________________________________________________________________

    This unit is a part of the SnakeASCII console game.
   __________________________________________________________________________

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
   __________________________________________________________________________

}

unit Converter;

{$MODE OBJFPC}{$H+}

interface

uses
  Common;


type
  TConverter = class(TObject)
  public
    class function SameModulesCoords(AFirst, ASecond: TModuleCoords): Boolean;
    class function CoordsRelationshipToMoveDirection(AEarlier, ALater: TModuleCoords): TMoveDirection;
    class function MoveDirectionsRelationshipToModuleBasicType(AEarlier, ALater: TMoveDirection): TSnakeModuleBasicType;
    class function MoveDirectionToModuleAdditionalType(ADirection: TMoveDirection): TSnakeModuleAdditionalType;
    class function SameDirection(ACurrDirection, ANewDirection: TMoveDirection): Boolean;
    class function OppositeDirection(ACurrDirection, ANewDirection: TMoveDirection): Boolean;
    class function MoveDirectionToModuleCoords(ADirection: TMoveDirection; ABaseCoords: TModuleCoords): TModuleCoords;
    class function CornerModule(AModuleType: TSnakeModuleBasicType): Boolean;
    class function InputKeyToMoveDirection(AInputKey: TInputKey): TMoveDirection;
  end;


{ ----- end interface ----------------------------------------------------------------------------------------- }


implementation


class function TConverter.SameModulesCoords(AFirst, ASecond: TModuleCoords): Boolean;
begin
  { compare coordinates }
  Result := (AFirst.X = ASecond.X) and (AFirst.Y = ASecond.Y);
end;


class function TConverter.CoordsRelationshipToMoveDirection(AEarlier, ALater: TModuleCoords): TMoveDirection;
const
  { snake horizontal and vertical move directions array }
  HORZ_MOVE_DIRECTION: array [Boolean] of TMoveDirection = (mdLeft, mdRight);
  VERT_MOVE_DIRECTION: array [Boolean] of TMoveDirection = (mdUp, mdDown);
begin
  if AEarlier.X = ALater.X then
    Result := VERT_MOVE_DIRECTION[AEarlier.Y > ALater.Y]
  else
    Result := HORZ_MOVE_DIRECTION[AEarlier.X > ALater.X];
end;


class function TConverter.MoveDirectionsRelationshipToModuleBasicType(AEarlier, ALater: TMoveDirection): TSnakeModuleBasicType;
begin
  { straight vertical }
  if ((AEarlier = mdUp)   and (ALater = mdUp)) or
     ((AEarlier = mdDown) and (ALater = mdDown)) then
    Exit(smbtVertical);

  { straight horizontal }
  if ((AEarlier = mdLeft)  and (ALater = mdLeft)) or
     ((AEarlier = mdRight) and (ALater = mdRight)) then
    Exit(smbtHorzizontal);

  { from top to left and bootom to right corners }
  if ((AEarlier = mdLeft) and (ALater = mdUp)) or
     ((AEarlier = mdDown) and (ALater = mdRight)) then
    Exit(smbtCornerLeftDown);

  { from top to right and bottom to left corners }
  if ((AEarlier = mdRight) and (ALater = mdUp)) or
     ((AEarlier = mdDown)  and (ALater = mdLeft)) then
    Exit(smbtCornerRightDown);

  { from bottom to left and top to right corners }
  if ((AEarlier = mdLeft) and (ALater = mdDown)) or
     ((AEarlier = mdUp)   and (ALater = mdRight)) then
    Exit(smbtCornerLeftUp);

  { from bottom to right and top to left corners }
  if ((AEarlier = mdRight) and (ALater = mdDown)) or
     ((AEarlier = mdUp)    and (ALater = mdLeft)) then
    Exit(smbtCornerRightUp);
end;


class function TConverter.MoveDirectionToModuleAdditionalType(ADirection: TMoveDirection): TSnakeModuleAdditionalType;
begin
  { check direction }
  case ADirection of
    mdUp:    Exit(smatTopHorz);
    mdDown:  Exit(smatBottomHorz);
    mdLeft:  Exit(smatLeftVert);
    mdRight: Exit(smatRightVert);
  end;
end;


class function TConverter.SameDirection(ACurrDirection, ANewDirection: TMoveDirection): Boolean;
begin
  { compare current direction and new direction }
  Result := ACurrDirection = ANewDirection;
end;


class function TConverter.OppositeDirection(ACurrDirection, ANewDirection: TMoveDirection): Boolean;
var
  mdMoves: set of TMoveDirection;
begin
  { compare directions }
  mdMoves := [ACurrDirection, ANewDirection];
  Result := (mdMoves = [mdUp, mdDown]) or (mdMoves = [mdLeft, mdRight]);
end;


class function TConverter.MoveDirectionToModuleCoords(ADirection: TMoveDirection; ABaseCoords: TModuleCoords): TModuleCoords;
begin
  { modify the base coordinates on the basis of move direction }
  case ADirection of
    mdUp:    Dec(ABaseCoords.Y);
    mdDown:  Inc(ABaseCoords.Y);
    mdLeft:  Dec(ABaseCoords.X);
    mdRight: Inc(ABaseCoords.X);
  end;

  { return the modified (or not) module coordinates }
  Exit(ABaseCoords);
end;


class function TConverter.CornerModule(AModuleType: TSnakeModuleBasicType): Boolean;
begin
  { compare module types }
  Result := AModuleType in [smbtCornerLeftUp, smbtCornerLeftDown, smbtCornerRightUp, smbtCornerRightDown];
end;

class function TConverter.InputKeyToMoveDirection(AInputKey: TInputKey): TMoveDirection;
begin
  { translate input key }
  case AInputKey of
    INPUT_DEFAULT_KEY_UP:    Exit(mdUp);
    INPUT_DEFAULT_KEY_DOWN:  Exit(mdDown);
    INPUT_DEFAULT_KEY_LEFT:  Exit(mdLeft);
    INPUT_DEFAULT_KEY_RIGHT: Exit(mdRight);
  else
    Exit(mdNone);
  end;
end;


{ ----- end implementation ------------------------------------------------------------------------------------ }


end.

