{

    Gold.pp                            last modified: 5 July 2014

    Copyright (C) Jaroslaw Baran, furious programming 2011 - 2014.
    All rights reserved.
   __________________________________________________________________________

    This unit is a part of the SnakeASCII console game.
   __________________________________________________________________________

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
   __________________________________________________________________________

}

unit Gold;

{$MODE OBJFPC}{$H+}

interface

uses
  Common;


type
  { single gold class }
  TGold = class(TObject)
  private
    FCoords: TModuleCoords;
    FDrawn: Boolean;
  private
    function RandomCoordsInRange(ALow, AHigh: UInt8): UInt8;
    function FarEnoughFromPrevious(ANewCoords: TModuleCoords): Boolean;
  public
    constructor Create();
    destructor Destroy(); override;
  public
    function IsGoldModule(ACoords: TModuleCoords): Boolean;
    procedure RandomNewCoords(AIsInLevel: TIsInLevelProc; AIsNotInSnake: TIsNotInSnakeProc);
  public
    property Coords: TModuleCoords read FCoords write FCoords;
    property Drawn: Boolean read FDrawn write FDrawn;
  end;


{ ----- end interface ----------------------------------------------------------------------------------------- }


implementation


constructor TGold.Create();
begin
  inherited Create();

  { initialize gold coordinates }
  FCoords.X := 1;
  FCoords.Y := 1;
  { initialize gold drawn state }
  FDrawn := False;
end;


destructor TGold.Destroy();
begin
  inherited Destroy();
end;


function TGold.RandomCoordsInRange(ALow, AHigh: UInt8): UInt8;
begin
  { random coordinate }
  repeat
    Result := Random(AHigh);
  until Result > ALow;
end;


function TGold.FarEnoughFromPrevious(ANewCoords: TModuleCoords): Boolean;
begin
  { compare previous and new gold coordinates }
  Result := (Abs(Integer(ANewCoords.X) - Integer(FCoords.X)) >= RANDOM_GOLD_MIN_DIFF) or
            (Abs(Integer(ANewCoords.Y) - Integer(FCoords.Y)) >= RANDOM_GOLD_MIN_DIFF);
end;


function TGold.IsGoldModule(ACoords: TModuleCoords): Boolean;
begin
  { compare modules coordinates }
  Result := (ACoords.X = FCoords.X) and (ACoords.Y = FCoords.Y);
end;


procedure TGold.RandomNewCoords(AIsInLevel: TIsInLevelProc; AIsNotInSnake: TIsNotInSnakeProc);
var
  mcNew: TModuleCoords;
begin
  { random new gold coordinates }
  repeat
    mcNew.X := RandomCoordsInRange(LBV_LEFT, LBV_RIGHT);
    mcNew.Y := RandomCoordsInRange(LBV_TOP, LBV_BOTTOM);
  until FarEnoughFromPrevious(mcNew) and AIsInLevel(mcNew) and AIsNotInSnake(mcNew);

  { update gold coordinates }
  FCoords := mcNew;
end;


{ ----- end implementation ------------------------------------------------------------------------------------ }


end.

