{

    Painter.pp                         last modified: 3 July 2014

    Copyright (C) Jaroslaw Baran, furious programming 2011 - 2014.
    All rights reserved.
   __________________________________________________________________________

    This unit is a part of the SnakeASCII console game.
   __________________________________________________________________________

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/>.
   __________________________________________________________________________

}

unit Painter;

{$MODE OBJFPC}{$H+}

interface

uses
  Crt, Converter, Level, Snake, Gold, Common;


type
  { main game elements painter class }
  TPainter = class(TObject)
  private
    FSnake: TSnake;
    FLevel: TLevel;
    FGold: TGold;
  private
    procedure DrawLevelBounds();
    procedure DrawGrassClumps();
  private
    function RandomGrassLine(AWidth: UInt8): AnsiString;
  public
    constructor Create();
    destructor Destroy(); override;
  public
    procedure PassSnake(ASnake: TSnake);
    procedure PassLevel(ALevel: TLevel);
    procedure PassGold(AGold: TGold);
  public
    procedure DrawLiveSnake();
    procedure DrawDeadSnake();
    procedure DrawLevel();
    procedure DrawGold();
    procedure ClearSnakeTail();
  public
    procedure DrawText(AX, AY, AColor: UInt8; AText: AnsiString);
    procedure ClearText(AX, AY, ATextWidth: UInt8);
  end;


{ ----- end interface ----------------------------------------------------------------------------------------- }


implementation


constructor TPainter.Create();
begin
  inherited Create();

  { initialize the snake and the level objects }
  FSnake := nil;
  FLevel := nil;
end;


destructor TPainter.Destroy();
begin
  { nil the snake and the level objects }
  FSnake := nil;
  FLevel := nil;

  inherited Destroy();
end;


procedure TPainter.PassSnake(ASnake: TSnake);
begin
  { store the snake object reference in field }
  FSnake := ASnake;
end;


procedure TPainter.PassLevel(ALevel: TLevel);
begin
  { store the level object reference in field }
  FLevel := ALevel;
end;


procedure TPainter.PassGold(AGold: TGold);
begin
  { store the pointer to gold coordinates record }
  FGold := AGold;
end;


procedure TPainter.DrawLiveSnake();
var
  smrToken: TSnakeModuleRec;
  mdBodyToTail: TMoveDirection;
  boolCornerModule, boolGateModule: Boolean;
begin
  { do not change the order of drawing modules of the snake! }

  { draw snake neck module }
  smrToken := FSnake[FSnake.Length - 2];
  { set the snake body color }
  boolGateModule := FSnake.IsGate(smrToken.Coords);
  boolCornerModule := TConverter.CornerModule(smrToken.ModuleType);
  TextColor(SNAKE_COLORATION_LIVE_SHADES_ARR[FLevel.SnakeColoration][(smrToken.GoldModule or boolGateModule) and not boolCornerModule]);
  { move cursor and draw snake neck }
  GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);

  { gate module state }
  if boolGateModule then
    { draw gate character }
    Write(AnsiChar(smatCrossGate))
  else
    { draw body character }
    Write(AnsiChar(smrToken.ModuleType));

  { draw snake tail module }
  smrToken := FSnake[0];
  boolGateModule := FSnake.IsGate(smrToken.Coords);
  { set the snake head color }
  TextColor(SNAKE_COLORATION_LIVE_HEAD_TAIL_SHADE);
  { move cursor and draw snake tail }
  GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);

  { gate state }
  if boolGateModule then
  begin
    { get the body and tail modules coordinates relationship }
    mdBodyToTail := TConverter.CoordsRelationshipToMoveDirection(FSnake[1].Coords, FSnake[0].Coords);
    { write the module type character on the basis of direction }
    Write(AnsiChar(TConverter.MoveDirectionToModuleAdditionalType(mdBodyToTail)));
  end
  else
    { draw body character }
    Write(AnsiChar(smrToken.ModuleType));

  { draw snake head module }
  smrToken := FSnake[FSnake.Length - 1];
  { set the snake head color }
  TextColor(SNAKE_COLORATION_LIVE_HEAD_TAIL_SHADE);
  { move cursor and draw snake head }
  GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);
  Write(Char(smrToken.ModuleType));
end;


procedure TPainter.DrawDeadSnake();
var
  smrToken: TSnakeModuleRec;
  slToken: TSnakeLength;
  mdBodyToTail: TMoveDirection;
  boolGateModule, boolCornerModule, boolSameAsHeadCoords: Boolean;
  intDelayTime: UInt16;
begin
  { calculate the frames delay time }
  case FSnake.Length of
    1  .. 10: intDelayTime := 500  div FSnake.Length; { for short snake length     }
    11 .. 20: intDelayTime := 1000 div FSnake.Length; { for medium snake length    }
    21 .. 30: intDelayTime := 1500 div FSnake.Length; { for long snake length      }
    31 .. 50: intDelayTime := 2000 div FSnake.Length; { for very long snake length }
  else
    { for spaghetti }
    intDelayTime := 3000 div FSnake.Length;
  end;

  { do not change the order of drawing modules of the snake! }

  { draw snake dead head module }
  smrToken := FSnake[FSnake.Length - 1];
  { set the snake head module color }
  TextColor(SNAKE_COLORATION_DEAD_HEAD_TAIL_SHADE);
  { move cursor and draw snake head }
  GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);
  Write(AnsiChar(smrToken.ModuleType));

  { sleep the frame }
  Delay(intDelayTime);

  { draw snake dead body modules in animated style }
  for slToken := FSnake.Length - 2 downto 1 do
  begin
    { draw snake dead body module }
    smrToken := FSnake[slToken];
    boolGateModule := FSnake.IsGate(smrToken.Coords);
    boolCornerModule := TConverter.CornerModule(smrToken.ModuleType);
    boolSameAsHeadCoords := TConverter.SameModulesCoords(smrToken.Coords, FSnake.HeadCoords);
    { set the snake body module color }
    TextColor(SNAKE_COLORATION_DEAD_SHADES_ARR[(smrToken.GoldModule or boolGateModule) and not boolCornerModule]);
    { move cursor and draw snake body module }
    GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);

    { same coordinates state }
    if not boolSameAsHeadCoords then
      { gate module state }
      if boolGateModule then
      begin
        { same current module coordinates as tail }
        if TConverter.SameModulesCoords(smrToken.Coords, FSnake[0].Coords) then
        begin
          { set the tail color insted of gold or gate color }
          TextColor(SNAKE_COLORATION_DEAD_HEAD_TAIL_SHADE);
          { get the body and tail modules coordinates relationship }
          mdBodyToTail := TConverter.CoordsRelationshipToMoveDirection(FSnake[1].Coords, FSnake[0].Coords);
          { write the module type character on the basis of direction }
          Write(AnsiChar(TConverter.MoveDirectionToModuleAdditionalType(mdBodyToTail)));
        end
        else
          { draw cross gate character }
          Write(AnsiChar(smatCrossGate));
      end
      else
        { same coordinates state }
        if not boolSameAsHeadCoords then
          { write body character }
          Write(AnsiChar(smrToken.ModuleType));

    { sleep the frame }
    Delay(intDelayTime);
  end;

  { set the snake tail module color }
  TextColor(SNAKE_COLORATION_DEAD_HEAD_TAIL_SHADE);
  { draw snake dead tail module }
  smrToken := FSnake[0];
  boolGateModule := FSnake.IsGate(smrToken.Coords);
  boolSameAsHeadCoords := TConverter.SameModulesCoords(smrToken.Coords, FSnake.HeadCoords);
  { move cursor and draw snake tail }
  GotoXY(smrToken.Coords.X + MCO_HORZ, smrToken.Coords.Y + MCO_VERT);

  { same coordinates state }
  if not boolSameAsHeadCoords then
    { gate module state }
    if boolGateModule then
    begin
      { draw 'T' gate character }
      { get the body and tail modules coordinates relationship }
      mdBodyToTail := TConverter.CoordsRelationshipToMoveDirection(FSnake[1].Coords, FSnake[0].Coords);
      { write the module type character on the basis of direction }
      Write(AnsiChar(TConverter.MoveDirectionToModuleAdditionalType(mdBodyToTail)));
    end
    else
      { draw body character }
      Write(AnsiChar(smrToken.ModuleType));
end;


procedure TPainter.DrawLevel();
begin
  { draw level bounds }
  DrawLevelBounds();
  { draw grass clumps }
  DrawGrassClumps();
end;


procedure TPainter.DrawLevelBounds();
var
  intX, intY: UInt8;
begin
  { set the level bounds color }
  TextColor(FLevel.BoundsColor);

  { draw level bounds in animated style (the empty modules is drawing too) }
  for intY := LBV_TOP to LBV_BOTTOM do
  begin
    GotoXY(LBV_LEFT + MCO_HORZ, intY + MCO_VERT);

    { sleep the frame }
    Delay(LEVEL_DRAWING_DELAY_TIME);

    { draw row }
    for intX := LBV_LEFT to LBV_RIGHT do
      Write(FLevel.ModuleChar[intX, intY]);
  end;
end;


procedure TPainter.DrawGrassClumps();
var
  lgcToken: TLevelGrassClumpRec;
  intToken, intY: Integer;
begin
  { draw grass clumps }
  for intToken := 0 to FLevel.ClumpsCount - 1 do
  begin
    { get grass clump record }
    lgcToken := FLevel.Clump[intToken];
    { set the grass clump color }
    TextColor(lgcToken.Color);

    { sleep the frame }
    Delay(LEVEL_DRAWING_DELAY_TIME);

    { draw grass clump }
    for intY := lgcToken.Top to lgcToken.Bottom do
    begin
      GotoXY(lgcToken.Left + MCO_HORZ, intY + MCO_VERT);
      Write(RandomGrassLine(lgcToken.Right - lgcToken.Left + 1));
    end;
  end;
end;


function TPainter.RandomGrassLine(AWidth: UInt8): AnsiString;
begin
  { set the length of result line }
  SetLength(Result, AWidth);

  { random grass characters }
  while AWidth > 0 do
  begin
    Result[AWidth] := GRASS_CLUMP_CHARS_ARR[Random(GRASS_CLUMP_CHARS_COUNT * 32) and %0011];
    Dec(AWidth);
  end;
end;


procedure TPainter.DrawGold();
begin
  { set the gold color }
  TextColor(GOLD_COLOR);
  { draw gold }
  GotoXY(FGold.Coords.X + MCO_HORZ, FGold.Coords.Y + MCO_VERT);
  Write(GOLD_SYMBOL);
end;


procedure TPainter.ClearSnakeTail();
var
  mcTail: TModuleCoords;
  boolGateModule: Boolean;
begin
  { get the tail module coordinates }
  mcTail := FSnake[0].Coords;
  { check gate in tail coordinates }
  boolGateModule := FSnake.IsGate(mcTail);

  { move cursor }
  GotoXY(mcTail.X + MCO_HORZ, mcTail.Y + MCO_VERT);

  { gate module state }
  if boolGateModule then
  begin
    { set the color for snake body module }
    TextColor(SNAKE_COLORATION_LIVE_SHADES_ARR[FLevel.SnakeColoration][False]);
    { draw snake body module }
    Write(AnsiChar(FSnake.ModuleType[mcTail]));
  end
  else
    { clear snake tail }
    Write(#32);
end;


procedure TPainter.DrawText(AX, AY, AColor: UInt8; AText: AnsiString);
begin
  { set the text color }
  TextColor(AColor);
  { move cursor }
  GotoXY(AX, AY);
  { draw text }
  Write(AText);
end;


procedure TPainter.ClearText(AX, AY, ATextWidth: UInt8);
begin
  { move cursor }
  GotoXY(AX, AY);
  { draw ATextWidth space characters }
  Write(StringOfChar(#32, ATextWidth));
end;


{ ----- end implementation ------------------------------------------------------------------------------------ }


end.

